#!/bin/sh
# Copyright 2013 IBM, Inc.
# Copyright 2017 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Refer to the README and COPYING files for full details of the license
#

VDSM_TOOL="/usr/bin/vdsm-tool"
UPGRADE_LOGGING_PARAMS="--vvverbose --append --logfile=/var/log/vdsm/upgrade.log"
prog=vdsm

#### pre-start tasks ####
task_wait_for_network() {
    # interfaces that we depend upon may not have their IP address ready yet
    # see https://bugzilla.redhat.com/show_bug.cgi?id=1174611#c34

    /usr/libexec/vdsm/wait_for_ipv4s

    # never fail, in order to allow intentionally-downed vdsm-controlled
    # interfaces
    return 0
}

task_configure_coredump() {
    echo "|/usr/libexec/abrt-hook-ccpp %s %c %p %u %g %t %e %i" > /proc/sys/kernel/core_pattern
}

task_configure_vdsm_logs() {
    local vdsm_logs="
        /var/log/vdsm/mom.log
        /var/log/vdsm/vdsm.log
    "

    for logfile in ${vdsm_logs}; do
        if [ ! -e "${logfile}" ]; then
            /usr/bin/touch "${logfile}"
            /usr/bin/chmod 0644 "${logfile}"
            /usr/bin/chown vdsm:kvm "${logfile}"
        fi
    done
}

task_run_init_hooks(){
    "/usr/bin/python" -m "vdsm.hooks" before_vdsm_start
}


task_check_is_configured() {
    "$VDSM_TOOL" is-configured
}


task_prepare_transient_repository() {
    "$VDSM_TOOL" setup-transient-repository
    "$VDSM_TOOL" cleanup-transient-repository
}


task_syslog_available() {
    if ! [ -S "/dev/log" ]; then
        echo "    Cannot communicate with syslog daemon for reports. "
        echo "    Install syslog daemon provider, configure it properly"
        echo "    and start the service."
        echo "    (Alternatives: rsyslog, syslog-ng)"
        return 1
    fi
    return 0
}


task_nwfilter(){
    "${VDSM_TOOL}" nwfilter
}


task_dummybr(){
    "${VDSM_TOOL}" dummybr
}


_has_systemd() {
    "/usr/bin/mountpoint" -q /cgroup/systemd ||
        "/usr/bin/mountpoint" -q /sys/fs/cgroup/systemd
}


task_dump_bonding_options(){
    "${VDSM_TOOL}" dump-bonding-options
}


task_tune_system(){
    "/usr/sbin/sysctl" -q -p "/etc/sysctl.d/vdsm.conf"
}


_CONF_FILE="/etc/vdsm/vdsm.conf"
_GETCONFITEM="/usr/libexec/vdsm/get-conf-item"
_DOM_METADATA_BACKUP_DIR="/var/log/vdsm/backup"

_mk_data_center() {
    local dc
    dc="$("${_GETCONFITEM}" "${_CONF_FILE}" irs repository /rhev/)"
    /usr/bin/mkdir -p "${dc}"
    "/usr/bin/chown" vdsm:kvm "${dc}"
}

_mk_dom_backup() {
    /usr/bin/mkdir -p "${_DOM_METADATA_BACKUP_DIR}" > /dev/null 2>&1
    "/usr/bin/chown" vdsm:kvm "$(dirname "${_DOM_METADATA_BACKUP_DIR}")" > /dev/null 2>&1
    "/usr/bin/chown" vdsm:kvm "${_DOM_METADATA_BACKUP_DIR}" > /dev/null 2>&1
}

_mk_core_path() {
    local core_path=/var/log/core
    if ! [ -d "${core_path}" ]; then
        /usr/bin/mkdir -p "${core_path}" > /dev/null 2>&1
    fi
    "/usr/bin/chmod" a+tw "${core_path}" > /dev/null 2>&1
}

_mk_run_path() {
    local run_path

    for run_path in "/var/run/vdsm"; do
        /usr/bin/mkdir -p "${run_path}"
        "/usr/bin/chmod" 755 "${run_path}"
        "/usr/bin/chown" vdsm:kvm "${run_path}"
        "/sbin/restorecon" "${run_path}"
    done
}

_mk_console_path() {
    local console_path=/var/run/ovirt-vmconsole-console
    if ! [ -d "${console_path}" ]; then
        /usr/bin/mkdir -p "${console_path}" > /dev/null 2>&1
    fi
    "/usr/bin/chown" vdsm:qemu "${console_path}"
    "/usr/bin/chmod" 0775 "${console_path}" > /dev/null 2>&1
    "/usr/bin/chcon" "system_u:object_r:qemu_var_run_t:s0" "${console_path}" > /dev/null 2>&1
}

task_mkdirs(){
    _mk_data_center
    _mk_core_path
    _mk_dom_backup
    _mk_run_path
    _mk_console_path
    "/usr/bin/chmod" 1777 /dev/shm
}


_free_space() {
    local pathToTest="$1"
    df -P "${pathToTest}" | awk '{print $4}'| tail -1
}

task_test_space() {
    local MIN_SPACE_KB=10000

    if [ "$(_free_space "/var/log/vdsm")" -lt "${MIN_SPACE_KB}" ]; then
        echo "${prog}: low log space"
        return 1
    fi
    return 0
}


task_test_lo() {
    if ! LC_ALL=C "/sbin/ip" link show lo | "/usr/bin/grep" -q UP; then
        echo "VDSMD: lo interface is down, can't run !"
        echo "VDSMD: lo interface is down, can't run !" > /dev/kmsg
        return 1
    fi
    return 0
}


task_validate_configuration(){
    "$VDSM_TOOL" validate-config
}


#### pre-start tasks end ####


#### post-stop tasks ####
task_run_final_hooks(){
    "/usr/bin/python" -m "vdsm.hooks" after_vdsm_stop
}
#### post-stop tasks end ####


usage(){
    echo "Usage:"
    echo "vdsm_init_common.sh --pre-start|--post-stop"
}

run_tasks(){
    local tasks="$1"
    for taskName in $tasks ; do
        echo "${prog}: Running ${taskName}"
        "task_${taskName}"
        rt=$?
        if [ "$rt" -ne 0 ]; then
            echo "${prog}: stopped during execute ${taskName} task" \
                 "(task returned with error code ${rt})."
            exit 1
        fi
    done
}

case "$1" in
    --pre-start)
        run_tasks " \
            mkdirs \
            configure_coredump \
            configure_vdsm_logs \
            wait_for_network \
            run_init_hooks \
            check_is_configured \
            validate_configuration \
            prepare_transient_repository \
            syslog_available \
            nwfilter \
            dummybr \
            tune_system \
            test_space \
            test_lo \
            "
        ;;
    --post-stop)
        run_tasks " \
            run_final_hooks \
            "
        ;;
    --help|*)
        usage
        exit 1
        ;;
esac
